{ config, pkgs, lib, ... }:

{
  imports =
    [
      ./hardware-configuration.nix
      ../../configuration/common
      ../../configuration/router
    ];

  boot.kernelParams = [ "console=ttyS0,115200n8" ];

  boot.loader.grub.enable = true;
  boot.loader.grub.version = 2;
  boot.loader.grub.device = "/dev/sda";
  boot.loader.grub.extraConfig = "
    serial --speed=115200 --unit=0 --word=8 --parity=no --stop=1
    terminal_input serial
    terminal_output serial
  ";

  networking.hostName = "carbon";

  networking.useDHCP = false;
  # Local Router IPs
  networking.interfaces.lo.ipv6.addresses = [
    { address = "fd00:152:152:104::1"; prefixLength = 64; }
    { address = "fd00:152:152::1"; prefixLength = 128; } # Anycast
  ];
  networking.interfaces.lo.ipv4.addresses = [
    { address = "10.152.104.1"; prefixLength = 24; }
    { address = "10.152.0.1"; prefixLength = 32; } # Anycast
  ];
  # Network
  ## DSL-Uplink
  networking.vlans."enp1s0.7" = {
    id = 7;
    interface = "enp1s0";
  };
  ## LTE-Uplink
  networking.vlans."enp1s0.102" = {
    id = 102;
    interface = "enp1s0";
  };
  ## Heimnetz
  networking.vlans."enp1s0.201" = {
    id = 201;
    interface = "enp1s0";
  };
  networking.interfaces."enp1s0.201".ipv6.addresses = [
    { address = "fd00:152:152:4::1"; prefixLength = 64; }
    { address = "2a01:4f8:1c0c:8221::1"; prefixLength = 64; } # public IPs for local network
  ];
  networking.interfaces."enp1s0.201".ipv4.addresses = [
    { address = "10.152.4.1"; prefixLength = 24; }
  ];
  ## Gastnetz
  networking.vlans."enp1s0.202" = {
    id = 202;
    interface = "enp1s0";
  };
  networking.interfaces."enp1s0.202".ipv6.addresses = [
    { address = "fd00:3214:9453:4920::1"; prefixLength = 64; }
  ];
  networking.interfaces."enp1s0.202".ipv4.addresses = [
    { address = "192.168.32.1"; prefixLength = 24; }
  ];
  ## VoIP
  networking.vlans."enp1s0.204" = {
    id = 204;
    interface = "enp1s0";
  };
  networking.interfaces."enp1s0.204".ipv4.addresses = [
    { address = "10.152.33.1"; prefixLength = 24; }
  ];
  # Use Anycast Nameservers
  networking.nameservers = [ "fd00:152:152::1" "10.152.0.1" ];

  networking.nat = {
    enableIPv6 = true;
    enable = true;
    externalInterface = "ppp-ntvdsl";
    internalIPv6s = [ "fd00:152:152::/48" "fd00:3214:9453:4920::/64"];
    internalIPs = [ "10.152.0.0/16" "192.168.32.0/24" ];
  };

  services.radvd.enable = true;
  services.radvd.config = ''
    interface enp1s0.201 {
      AdvSendAdvert on;
      prefix 2a01:4f8:1c0c:8221::/64 {};
      route ::/0 {};
      RDNSS fd00:152:152::1 {};
      DNSSL net.clerie.de {};
    };
  '';

  services.kea.dhcp4 = {
    enable = true;
    settings = {
      interfaces-config = {
        interfaces = [ "enp1s0.201" "enp1s0.204" ];
      };
      lease-database = {
        name = "/var/lib/kea/dhcp4.leases";
        persist = true;
        type = "memfile";
      };
      option-def = [
        {
          space = "dhcp4";
          name = "vendor-encapsulated-options";
          code = 43;
          type = "empty";
          encapsulate = "sipdect";
        }
        {
          space = "sipdect";
          name = "ommip1";
          code = 10;
          type = "ipv4-address";
        }
        {
          space = "sipdect";
          name = "ommip2";
          code = 19;
          type = "ipv4-address";
        }
        {
          space = "sipdect";
          name = "syslogip";
          code = 14;
          type = "ipv4-address";
        }
        {
          space = "sipdect";
          name = "syslogport";
          code = 15;
          type = "int16";
        }
        {
          space = "dhcp4";
          name = "magic_str";
          code = 224;
          type = "string";
        }
      ];
      subnet4 = [
        # Heimnetz
        {
          id = 201;
          subnet = "10.152.4.0/24";
          pools = [
            {
              pool = "10.152.4.100 - 10.152.4.240";
            }
          ];
          option-data = [
            {
              name = "routers";
              data = "10.152.4.1";
            }
            {
              name = "domain-name-servers";
              data = "10.152.0.1";
            }
            {
              name = "domain-name";
              data = "net.clerie.de";
            }
          ];
        }
        # VoIP
        {
          id = 204;
          subnet = "10.152.33.0/24";
          pools = [
            {
              pool = "10.152.33.10 - 10.152.33.200";
            }
          ];
          option-data = [
            {
              name = "routers";
              data = "10.152.33.1";
            }
          ];

          reservations = [
            {
              hostname = "iridium";
              hw-address = "00:30:42:1B:8C:7C";
              ip-address = "10.152.33.11";
              option-data = [
                {
                  name = "host-name";
                  data = "iridium";
                }
                {
                  name = "vendor-encapsulated-options";
                }
                {
                  space = "sipdect";
                  name = "ommip1";
                  data = "10.152.33.11";
                }
                {
                  name = "magic_str";
                  data = "OpenMobilitySIP-DECT";
                }
              ];
            }
          ];
        }
      ];
    };
  };

  networking.wireguard.enable = true;
  networking.wireguard.interfaces = {
    wg-gatekeeper4 = {
      ips = [ "fe80::127:2/64" "169.254.127.2/24" ];
      peers = [ {
          allowedIPs = [ "0.0.0.0/0" "::/0" ];
          endpoint = "78.47.183.82:50127";
          publicKey = "y+Bk5eIHgmnq9xuBDD+fk/OIkKRZU6AE4ISx4RdDDyg=";
          persistentKeepalive = 25;
      } ];
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-gatekeeper4";
    };
    wg-porter4 = {
      ips = [ "fe80::138:2/64" "169.254.138.2/24" ];
      peers = [ {
          allowedIPs = [ "0.0.0.0/0" "::/0" ];
          endpoint = "5.45.100.191:50138";
          publicKey = "aP6optNE7nVk6coo+USkSDtB62rAc/isfofRML9V2HM=";
          persistentKeepalive = 25;
      } ];
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-porter4";
    };
    wg-heimnetz = {
      ips = [ "fd00:153:153:201::2/64" ];
      peers = [ {
        allowedIPs = [ "::/0" ];
        endpoint = "[fd00:152:152:101::1]:60001";
        publicKey = "j/XAIOJGgLieg0jry4AGSkxQySuDdwhJShqC5SCgsWw=";
      } ];
      listenPort = 60001;
      allowedIPsAsRoutes = false;
      privateKeyFile = "/var/src/secrets/wireguard/wg-heimnetz";
      postSetup = "ip link set wg-heimnetz mtu 1340";
    };
  };

  networking.firewall.allowedUDPPorts = [ 53 60001 ];

  networking.firewall.extraCommands = ''
    iptables -A FORWARD -i enp1s0.202 -o ppp-ntvdsl -j ACCEPT
    iptables -A FORWARD -i enp1s0.202 -j DROP
    iptables -A FORWARD -i ppp-ntvdsl -o enp1s0.202 -j ACCEPT
    iptables -A FORWARD -o enp1s0.202 -j DROP
  '';

  services.pppd = {
    enable = true;
    peers.ntvdsl = {
      config = ''
        plugin rp-pppoe.so enp1s0.7
        user "dsl-31997-001#regio@bsa-vdsl"
        ifname ppp-ntvdsl
        persist
        maxfail 0
        holdoff 5
        noipdefault
        lcp-echo-interval 20
        lcp-echo-failure 3
        mtu 1492
        hide-password
        defaultroute
        +ipv6
        debug
      '';
    };
  };


  # Routing tables
  # Table: 10000
  # - primary routes
  # Table: 11000
  # - ospf routes
  # Table: 20101
  # - default route to gatekeeper
  #
  # We will never use main table anymore
  petabyte.policyrouting = {
    enable = true;
    rules6 = [
      # main routes first except default route
      { rule = "lookup main suppress_prefixlength 0"; prio = 10000; }
      # Prefixes defaulting to gatekeeper
      { rule = "from 2a01:4f8:1c0c:8221::/64 lookup 20101"; prio = 20000; }
      { rule = "from 2a01:4f8:1c0c:8221::/64 unreachable"; prio = 20001; }
      # Everything else defaulting to main table after this
    ];
    rules4 = [
      # main routes first except default route
      { rule = "lookup main suppress_prefixlength 0"; prio = 10000; }
      # Prefixes defaulting to gatekeeper
      #{ rule = "from xxx lookup 20101"; prio = 20000; }
      # Everything else defaulting to main table after this
    ];
  };

  services.bird2.enable = true;
  services.bird2.config = ''
  router id ${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };

  ipv6 table gatekeeper6;
  ipv4 table gatekeeper4;

  protocol static static_gatekeeper_6 {
          ipv6 {
                  table gatekeeper6;
          };
          route ::/0 via fd00:153:153:201::1;
  }
  protocol static static_gatekeeper_4 {
          ipv4 {
                  table gatekeeper4;
          };
          route 0.0.0.0/0 via 10.153.201.1;
  }

  protocol kernel kernel_gatekeeper_6 {
    ipv6 {
      table gatekeeper6;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv6.addresses).address };
        accept;
      };
      import none;
    };
    kernel table 20101;
  }
  protocol kernel kernel_gatekeeper_4 {
    ipv4 {
      table gatekeeper4;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };
        accept;
      };
      import none;
    };
    kernel table 20101;
  }

  ipv6 table ospf6;
  ipv4 table ospf4;

  protocol direct direct_lo {
    interface "lo";
    ipv6 {
      table ospf6;
    };
    ipv4 {
      table ospf4;
    };
  }

  protocol direct direct_enp1s0_201 {
    interface "enp1s0.201";
    ipv6 {
      table ospf6;
    };
    ipv4 {
      table ospf4;
    };
  }

  protocol kernel kernel_ospf6 {
    ipv6 {
      table ospf6;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv6.addresses).address };
        accept;
      };
      import none;
    };
  }

  protocol kernel kernel_ospf4 {
    ipv4 {
      table ospf4;
      export filter {
        krt_prefsrc=${ (lib.head config.networking.interfaces.lo.ipv4.addresses).address };
        accept;
      };
      import none;
    };
  }

  protocol ospf v3 ospf_6 {
    ipv6 {
      table ospf6;
      import all;
      export all;
    };
    area 0 {
      interface "wg-gatekeeper4" {
        cost 80;
        type pointopoint;
      };
      interface "wg-porter4" {
        cost 80;
        type pointopoint;
      };
    };
  }

  protocol ospf v3 ospf_4 {
    ipv4 {
      table ospf4;
      import all;
      export all;
    };
    area 0 {
      interface "wg-gatekeeper4" {
        cost 80;
        type pointopoint;
      };
      interface "wg-porter4" {
        cost 80;
        type pointopoint;
      };
    };
  }

  protocol device {
    scan time 10;
  }
  '';

  clerie.monitoring = {
    enable = true;
    id = "104";
    pubkey = "sro9DUSMtVr5xV2o3GTgg+0vmLj+bRc8fN+3pIr6+HY=";
  };

  services.unbound = {
    enable = true;
    resolveLocalQueries = false;
    settings = {
      server = {
        interface = [ "fd00:152:152::1" "10.152.0.1" ];
        access-control = [ "::/0 allow" "0.0.0.0/0 allow" ];
        prefer-ip6 = true;
        prefetch = true;
        serve-expired = true;
        serve-expired-ttl-reset = true;
      };
    };
  };

  system.stateVersion = "21.03";
}
